import './index.scss'
import Input from './Input'
import Select from './Select'

export default {
  name: 'FzProForm',
  props: {
    options: {
      type: Object,
      default: () => ({})
    }
  },
  components: {
    Input,
    Select
  },

  data() {
    return {
      models: null
    }
  },

  created() {
    this.models = this.getModules()
  },

  methods: {
    getModules() {
      const modules = {}
      const lists = this.options.list
      for (const key in lists) {
        const formItems = lists[key].formItems
        for (const moduleKey in formItems) {
          modules[moduleKey] = formItems[moduleKey]?.options?.value || ''
        }
      }

      return modules
    },

    async submitForm() {
      const valid = await this.$refs.formRef.validate()

      if (valid) {
        console.log(this.models, 'models')
      }
    },

    resetForm() {
      this.$refs.formRef.resetFields()
    }
  },

  render() {
    const { options, models, $slots, $scopedSlots, submitForm, resetForm } =
      this

    // 表单Item 默认展示组件
    const getDefaultFormSlots = (key, form) => {
      const VNodes = {
        Input: (
          <Input
            options={{ formOptions: form.options, models }}
            modelKey={key}
          />
        ),
        Select: (
          <Select
            options={{ formOptions: form.options, models }}
            modelKey={key}
          />
        )
      }

      return VNodes[form.type]
    }

    const formItemRender = (formItems) => {
      const formItemList = Object.entries(formItems)

      return formItemList.map((item) => {
        const [key, form] = item

        return (
          <el-row>
            <el-col props={form.formItemOptions.col} key={key}>
              {!form?.formItemOptions?.isHidden && (
                <el-form-item prop={key} props={form.formItemOptions}>
                  <el-row>
                    <el-col props={form.options.col}>
                      {/* 表单插槽 */}
                      {$scopedSlots?.[key]?.({
                        item: form,
                        data: models[key]
                      }) || getDefaultFormSlots(key, form)}
                    </el-col>

                    {/* Form-item 后缀 */}
                    {$scopedSlots[`${key}-after`]?.({
                      data: form
                    }) ||
                      (form.formItemOptions?.slotAfter && (
                        <span
                          class="slot-after"
                          style="margin-left: 10px; float: left"
                        >
                          {form.formItemOptions.slotAfter || null}
                        </span>
                      ))}
                  </el-row>
                </el-form-item>
              )}
            </el-col>
          </el-row>
        )
      })
    }

    // 表单Item渲染
    const formRender = () => {
      const entriesList = Object.entries(options.list)

      return entriesList.map((item, index) => {
        const [key, form] = item

        return (
          <el-col key={index} props={form.formCol}>
            {/* 表单区域标题 */}
            {$slots?.[key] || (
              <div class="form-title">
                {form?.formTitleIcon
                  ? $slots[`${key}-icon`] || (
                      <i class={['form-icon', form.formTitleIcon]} />
                    )
                  : ''}
                <span>{form.formTitle}</span>
              </div>
            )}
            {/* 数据表单 */}
            {formItemRender(form.formItems)}
          </el-col>
        )
      })
    }

    return (
      <div class="fz-pro__form">
        <el-form
          ref="formRef"
          {...{
            props: {
              model: models,
              ...options.config
            }
          }}
        >
          <el-row>{formRender()}</el-row>
        </el-form>

        {/* Submit Btn */}
        <div class="fz-pro__buttons">
          <el-button type="primary" onClick={submitForm}>
            提交
          </el-button>
          <el-button type="info" onClick={resetForm}>
            重置
          </el-button>
        </div>
      </div>
    )
  }
}
