import './index.scss'
import FzPagination from '../pagination'

export default {
  name: 'FzProTable',
  components: {
    FzPagination
  },
  props: {
    // 表格配置
    tableConfig: {
      type: Object,
      default: () => ({
        headerCellStyle: {
          backgroundColor: '#F2F3F5',
          color: '#212529',
          fontSize: '14px',
          border: 'none'
        }
      })
    },
    // 表格事件
    tableEvents: {
      type: Object,
      default: () => {}
    },
    // 表格数据
    dataList: {
      type: Array,
      default: () => []
    },
    // 数据请求
    request: {
      type: Function,
      default: () => {}
    },
    // 表格列
    columns: {
      type: Array,
      default: () => []
    },
    // 是否显示分页
    isPagination: {
      type: Boolean,
      default: true
    }
  },
  data() {
    return {
      requestData: {},
      pages: {
        currentPage: 1,
        pageSize: 10
      },
      loading: true
    }
  },
  computed: {
    getTableDefaultStyle() {
      return {
        backgroundColor: '#F2F3F5',
        color: '#212529',
        fontSize: '14px',
        border: 'none'
      }
    },
    // 在 Table 中不展示此列 用户表单搜索
    filterColumns({ columns }) {
      return columns
        .filter((item) => !item?.hideInTable)
        .sort((a, b) => (b?.tableOrder || 0) - (a?.tableOrder || 0))
    }
  },
  created() {
    this.getRequestTableData(this.pages)
  },

  methods: {
    // 获取数据
    async getRequestTableData(params) {
      try {
        const data = await this.request(params)
        this.requestData = data
        this.loading = false
      } catch {
        this.loading = false
      }
    }
  },

  render() {
    const {
      getTableDefaultStyle,
      tableConfig,
      tableEvents,
      loading,
      requestData,
      $scopedSlots,
      $slots,
      filterColumns
    } = this
    const { data } = requestData

    // 分页触发事件
    const onPagination = (pages) => {
      this.pages = pages

      this.getRequestTableData(pages)
    }

    const getTableColumns = (columns) => {
      return columns.map((item) => {
        // 隐藏列
        if (item.hideInTable) return null

        // 超出提示框class
        const showOverflowTooltipClass = item['show-overflow-tooltip']
          ? 'cell el-tooltip'
          : 'cell'

        const scopedSlots = {
          default: (data) =>
            $scopedSlots[`${item.prop}-default`]?.(data) || (
              <div class={showOverflowTooltipClass}>
                {data.row[`${item.prop}`]}
              </div>
            ),
          header: (data) =>
            $scopedSlots[`${item.prop}-header`]?.(data) || (
              <span class={showOverflowTooltipClass}>{item.label}</span>
            )
        }

        switch (item?.type) {
          case 'expand':
            return (
              <el-table-column
                props={item}
                {...{
                  scopedSlots: {
                    default: (data) => $scopedSlots[`expand-default`](data)
                  }
                }}
                key={item.type}
              ></el-table-column>
            )
          case 'selection':
            return <el-table-column props={item} key={item.type} />
          case 'index':
            return <el-table-column props={item} key={item.type} />
          default:
            return (
              <el-table-column
                props={item}
                scopedSlots={scopedSlots}
                key={item.type}
              />
            )
        }
      })
    }

    return (
      <div class="fz-table__wrapper">
        <div class="fz-table__content">
          <el-table
            ref="tableRef"
            headerCellStyle={getTableDefaultStyle}
            data={data?.records}
            props={tableConfig}
            on={tableEvents}
            vLoading={loading}
          >
            {getTableColumns(filterColumns)}

            <template slot="append">{$slots.append}</template>
            {/* TODO: Empty组件封装 */}
            <template slot="empty">
              {'暂无数据'}
              {/* <Empty /> */}
            </template>
          </el-table>
        </div>

        {/* 分页区域 */}
        {this.isPagination ? (
          <div class="fz-table__pagination">
            <fz-pagination onPagination={onPagination} total={data?.total} />
          </div>
        ) : null}
      </div>
    )
  }
}
